package Data::Sah::Lang::zh_CN;

our $DATE = '2015-05-13'; # DATE
our $VERSION = '0.56'; # VERSION

use 5.010;
use strict;
use warnings;

use Tie::IxHash;

# currently incomplete

our %translations;
tie %translations, 'Tie::IxHash', (

    # punctuations

    q[ ], # inter-word boundary
    q[],

    q[, ],
    q[ÃÂ¯ÃÂ¼ÃÂ],

    q[: ],
    q[ÃÂ¯ÃÂ¼ÃÂ],

    q[. ],
    q[ÃÂ£ÃÂÃÂ],

    q[(],
    q[ÃÂ¯ÃÂ¼ÃÂ],

    q[)],
    q[ÃÂ¯ÃÂ¼ÃÂ],

    # modal verbs

    q[must],
    q[ÃÂ¥ÃÂ¿ÃÂÃÂ©ÃÂ¡ÃÂ»],

    q[must not],
    q[ÃÂ¥ÃÂ¿ÃÂÃÂ©ÃÂ¡ÃÂ»ÃÂ¤ÃÂ¸ÃÂ],

    q[should],
    q[ÃÂ¥ÃÂºÃÂ],

    q[should not],
    q[ÃÂ¥ÃÂºÃÂÃÂ¤ÃÂ¸ÃÂ],

    # field/fields/argument/arguments

    q[field],
    q[ÃÂ¥ÃÂ­ÃÂÃÂ¦ÃÂ®ÃÂµ],

    q[fields],
    q[ÃÂ¥ÃÂ­ÃÂÃÂ¦ÃÂ®ÃÂµ],

    q[argument],
    q[ÃÂ¥ÃÂÃÂÃÂ¦ÃÂÃÂ°],

    q[arguments],
    q[ÃÂ¥ÃÂÃÂÃÂ¦ÃÂÃÂ°],

    # multi

    q[%s and %s],
    q[%sÃÂ¥ÃÂÃÂ%s],

    q[%s or %s],
    q[%sÃÂ¦ÃÂÃÂ%s],

    q[one of %s],
    q[ÃÂ¨ÃÂ¿ÃÂÃÂ¤ÃÂºÃÂÃÂ¥ÃÂÃÂ¼%sÃÂ¤ÃÂ¹ÃÂÃÂ¤ÃÂ¸ÃÂ],

    q[all of %s],
    q[ÃÂ¦ÃÂÃÂÃÂ¦ÃÂÃÂÃÂ¨ÃÂ¿ÃÂÃÂ¤ÃÂºÃÂÃÂ¥ÃÂÃÂ¼%s],

    q[%(modal_verb)s satisfy all of the following],
    q[%(modal_verb)sÃÂ¦ÃÂ»ÃÂ¡ÃÂ¨ÃÂ¶ÃÂ³ÃÂ¦ÃÂÃÂÃÂ¦ÃÂÃÂÃÂ¨ÃÂ¿ÃÂÃÂ¤ÃÂºÃÂÃÂ¦ÃÂÃÂ¡ÃÂ¤ÃÂ»ÃÂ¶],

    q[%(modal_verb)s satisfy one of the following],
    q[%(modal_verb)sÃÂ¦ÃÂ»ÃÂ¡ÃÂ¨ÃÂ¶ÃÂ³ÃÂ¨ÃÂ¿ÃÂÃÂ¤ÃÂºÃÂÃÂ¦ÃÂÃÂ¡ÃÂ¤ÃÂ»ÃÂ¶ÃÂ¤ÃÂ¹ÃÂÃÂ¤ÃÂ¸ÃÂ],

    q[%(modal_verb)s satisfy none of the following],
    q[%(modal_verb_neg)sÃÂ¦ÃÂ»ÃÂ¡ÃÂ¨ÃÂ¶ÃÂ³ÃÂ¦ÃÂÃÂÃÂ¦ÃÂÃÂÃÂ¨ÃÂ¿ÃÂÃÂ¤ÃÂºÃÂÃÂ¦ÃÂÃÂ¡ÃÂ¤ÃÂ»ÃÂ¶],

    # type: BaseType

    # type: Sortable

    # type: Comparable

    # type: HasElems

    # type: num

    # type: int

    q[integer],
    q[ÃÂ¦ÃÂÃÂ´ÃÂ¦ÃÂÃÂ°],

    q[integers],
    q[ÃÂ¦ÃÂÃÂ´ÃÂ¦ÃÂÃÂ°],

    q[%(modal_verb)s be divisible by %s],
    q[%(modal_verb)sÃÂ¨ÃÂ¢ÃÂ«%sÃÂ¦ÃÂÃÂ´ÃÂ©ÃÂÃÂ¤],

    q[%(modal_verb)s leave a remainder of %2$s when divided by %1$s],
    q[ÃÂ©ÃÂÃÂ¤ÃÂ¤ÃÂ»ÃÂ¥%1$sÃÂ¦ÃÂÃÂ¶ÃÂ¤ÃÂ½ÃÂÃÂ¦ÃÂÃÂ°%(modal_verb)sÃÂ¤ÃÂ¸ÃÂº%2$s],

);

1;
# ABSTRACT: zh_CN locale

__END__

=pod

=encoding UTF-8

=head1 NAME

Data::Sah::Lang::zh_CN - zh_CN locale

=head1 VERSION

This document describes version 0.56 of Data::Sah::Lang::zh_CN (from Perl distribution Perinci-CmdLine-Any-Bundled version 0.02 version 0.01 version 0.01), released on 2015-05-13.

=for Pod::Coverage .+

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Data-Sah>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Data-Sah>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Data-Sah>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
