package Perinci::CmdLine::Any::Bundled;

our $VERSION = "0.01";

1;
# ABSTRACT: Perinci::CmdLine::Any (bundled dist)

=head1 NAME

Perinci::CmdLine::Any::Bundled - Perinci::CmdLine::Any (bundled dist)


=head1 VERSION

0.01


=head1 DESCRIPTION

This distribution contains L<Perinci::CmdLine::Any> as well as some (unindexed)
deps bundled inside it. You can depend on Perinci::CmdLine::Any::Bundled instead
of Perinci::CmdLine::Any for fewer deps and faster CPAN installation.


=head1 SEE ALSO

L<Perinci::CmdLine::Lite>

L<Perinci::CmdLine::Classic>

=cut
